<?php
namespace Ep\Gath\Hydrator;

use Ep\Gath\Entity\Discount;
use Ep\Gath\Entity\Rebate;
use Interop\Container\ContainerInterface;
use Zend\Hydrator\HydratorInterface;
use Zend\Hydrator\HydratorPluginManager;
use Zend\Hydrator\NamingStrategy\MapNamingStrategy;
use Zend\Hydrator\Reflection as ReflectionHydrator;
use Zend\Hydrator\Strategy\ClosureStrategy;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu hidratora za Price objekt.
 *
 * @package Ep\Gath\Hydrator
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class PriceHydratorFactory implements FactoryInterface
{
    /**
     * Stvara objekt
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return object
     *
     * @throws \Exception
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
	public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
	{
		/** @var HydratorPluginManager $hydrators */
		$hydrators = $container->get('HydratorManager');
		$hydrator = new ReflectionHydrator();
		/** @var HydratorInterface $discountHydrator */
		$discountHydrator = $hydrators->get(Discount::class);
		/** @var HydratorInterface $rebateHydrator */
		$rebateHydrator = $hydrators->get(Rebate::class);

		$hydrator->setNamingStrategy(new MapNamingStrategy([
			'ProductCode' => 'sku',
			'TaxRate' => 'taxRate',
			'PriceRetail' => 'priceRetail',
			'PriceWholesale' => 'priceWholesale',
			'Discount' => 'discounts',
			'RabPolicy' => 'rebate',
            'HidePrice' => 'price_hidden',
		]));

		$hydrator->addStrategy('discounts', new ClosureStrategy(
			// extract
			function() {
				// za sada ne trebamo izvlačiti podatke iz entiteta
				return [];
			},
			// hydrate
			function($discounts) use ($discountHydrator) {
				$hydrated = [];

				foreach ($discounts as $discount) {
					$hydrated[] = $discountHydrator->hydrate($discount, new Discount());
				}

				return $hydrated;
			}
		));

		$hydrator->addStrategy('rebate', new ClosureStrategy(
			// extract
			function() {
				// za sada ne trebamo izvlačiti podatke iz entiteta
				return [];
			},
			// hydrate
			function($rebates) use ($rebateHydrator) {
				return is_array($rebates) && count($rebates) > 0
					? $rebateHydrator->hydrate($rebates[0], new Rebate())
					: null;
			}
		));

		return $hydrator;
	}
}